;       Compile with WLA-6510 (http://www.hut.fi/~vhelin/wla.html)

.INCLUDE "vic.inc"

*
*
*       CMON - Machine Language Monitor for Commodore VIC-20 - v1.0
*
*       (c)2001 Aleksi Eeben (aleksi@cncd.fi)
*       http://www.cncd.fi/aeeben
*
*

.DEFINE COMPILE_LITE    0               ; 1 = Compile LITE version
.DEFINE UNDOCUMENTED    1               ; 1 = 6502/6510 undocumented commands
.DEFINE COMPILE_C64     0               ; 1 = Target C64 instead of VIC-20

.DEFINE LoadAddr        $0400

.ORG $0
.SECTION "BANKHEADER"
        .dw     LoadAddr
.ENDS

.DEFINE CHRIN           $ffcf
.DEFINE CHROUT          $ffd2
.DEFINE SAVE            $ffd8
.DEFINE LOAD            $ffd5
.DEFINE SETLFS          $ffba
.DEFINE SETNAM          $ffbd
.DEFINE OPEN            $ffc0
.DEFINE CLOSE           $ffc3
.DEFINE CHKIN           $ffc6
.DEFINE READST          $ffb7
.DEFINE CLALL           $ffe7
.DEFINE STOP            $ffe1

.IF COMPILE_C64 == 0
.DEFINE DECOUT          $ddd1           ; BASIC ROM, print number in $62-$63
.DEFINE HexPerLine      4
.DEFINE TextLineWidth   16
.ELSE
.DEFINE DECOUT          $bdd1           ; BASIC ROM, print number in $62-$63
.DEFINE HexPerLine      8
.DEFINE TextLineWidth   32
.ENDIF


.ENUM $f7
Aux1    DB                              ; 8 zero page locations: $f7-$fe
Aux2    DB
Aux3    DB
Aux4    DB
Aux5    DB
Aux6    DB

CurrL   DB
CurrH   DB
.ENDE


.DEFINE HexDumpLines    4               ; M dumps 4 lines
.DEFINE TextViewLines   8               ; I dumps 8 lines
.DEFINE DisasmCount     16              ; D disassembles 16 instructions

.DEFINE AsmBuff         $0200           ; input buffer for A
.DEFINE Filename        $0200           ; 16 bytes for filename
.DEFINE Regs            $0253           ; 5 bytes for captured registers

.ORG LoadAddr
.SECTION "MAIN" FORCE


;****   Initialize monitor

Monitor:
        jsr     Capture                 ; capture registers

        lda     #LoadAddr/256           ; set up 'current' address
        sta     CurrH

        ldy     #$00                    ; welcome message id
        sty     CurrL
        beq     _continue               ; always branch


;****   Command line input

CmdLineInput:
        jsr     CHRIN

        cmp     #46                     ; skip dot at beginning of line
        beq     CmdLineInput            ; (if using the screen editor)

        cmp     #88                     ; "X" for exit
        beq     Exit

        jsr     CheckCmd

        lda     #$01                    ; ready message id 1, error id 2
        adc     #$00                    ; carry is set if error occurred
        tay
_continue:
        jsr     PrintMsg
        jmp     CmdLineInput


;****   Check and execute command

CheckCmd:
        ldy     #CmdJumpL-CmdList-1     ; # of commands in CmdList
_cmdcheck:
        cmp     CmdList,y
        beq     CmdFound
        dey
        bpl     _cmdcheck

        sec                             ; unknown command
        rts

CmdFound:
        lda     CmdJumpL,y
        sta     Aux5
        lda     CmdJumpH,y
        sta     Aux6

        jmp     (Aux5)                  ; execute command (rts there)


;****   Command X: Exit

Exit:
        lda     #$00                    ; clean up basic input buffer
        ldy     #$58

_clrbas:
        sta     $0200,y
        dey
        bpl     _clrbas

        rts


;****   Command A: Assemble

Assemble:
        jsr     GetAddr                 ; line: "A 1000 LDA #$00" or "A 1000"
        bcs     _asmout                 ; address required

        jsr     CheckSpace
        bcc     _asmfirst               ; "A 1000 LDA #$00" (addr + assembly)


;*      Input and buffer assembly instruction

_assemlp:
        jsr     CurrOut                 ; output: "1000 "
        jsr     SpaceOut

_asmfirst:
        ldy     #$00

        jsr     CHRIN                   ; return to exit
        cmp     #13
        bne     _notend
_endasm:
        clc                             ; no error
        rts

_asmbufflp:
        jsr     CHRIN                   ; read rest of line to buffer
_notend:
        sta     AsmBuff,y
        iny

        cmp     #13
        bne     _asmbufflp


;*      Recognize mnemonic (first 3 characters in buffer)

        ldy     #$00
_iseeklp:
        lda     AsmBuff                 ; compare 1st characters
        cmp     Assembly,y
        bne     _inomatch

        lda     AsmBuff+1               ; compare 2nd characters
        cmp     Assembly+1,y
        bne     _inomatch

        lda     AsmBuff+2               ; compare 3rd characters
        cmp     Assembly+2,y
        beq     _instrfound

_inomatch:
        iny                             ; next mnemonic
        iny
        iny
        cpy     #AddrSizes-Assembly     ; size of mnemonic table
        bne     _iseeklp                ; scan forward
        ;
_asmerr:
        sec                             ; error occurred
_asmout:
        rts

_instrfound:
        sty     Aux1                    ; store mnemonic pointer


;*      Recognize addressing mode

        ldy     #$0b
_amseeklp:
        ldx     #$00
        tya
        pha

        lda     AsmMaskPtr,y
        tay

_amcomp:
        lda     AsmMasks,y
        cmp     #$30                    ; match "0"
        beq     _ahexmatch
        
        cmp     AsmBuff+3,x
        bne     _amnomatch

        cmp     #13
        beq     _amfound

_amcmatch:
        iny                             ; matching so far
        inx
        bne     _amcomp                 ; always branch (next character)

_ahexmatch:
        lda     AsmBuff+3,x             ; "0" matches any char above $30
        cmp     #$30
        bcs     _amcmatch
        bcc     _amnomatch              ; always branch

_amnomatch:
        pla                             ; no addressing mode match
        tay
        dey
        bpl     _amseeklp               ; seek forward

        bmi     _asmerr                 ; no match error (always branch)

_amfound:
        pla                             ; actual addressing mode from stack
        sta     Aux2                    ; store for later use


;*      Find opcode with matching mnemonic and addressing mode

        ldy     #$00
_afindopcode:
        lda     MnemPtr,y               ; test for mnemonic
        cmp     Aux1
        bne     _anoopmatch

        lda     AddrModes,y             ; then test for addressing mode
        cmp     Aux2
        beq     _afoundop

        cmp     #$04                    ; also match relative and absolute
        bne     _anoopmatch             ; addressing modes which both
        lda     Aux2                    ; appear as "$0000"
        cmp     #$09
        beq     _afoundop               ; (kludge award)

_anoopmatch:
        iny
        cpy     #AddrModes-MnemPtr
        bne     _afindopcode            ; try next opcode

        beq     _asmerr                 ; no matching opcode (always branch)


;*      Write opcode to memory

_afoundop:
        ldx     #$00
        lda     Opcodes,y               ; get opcode from table
        sta     (CurrL,x)               ; write opcode

        lda     AddrModes,y             ; get actual addressing mode
        tay
        lda     AddrSizes,y             ; get instruction size

        cpy     #$00
        beq     _adone                  ; addressing mode 0 has no operand


;*      Write operand to memory

        sta     Aux1                    ; store instruction size

        ldx     #$00                    ; seek for "$" in AsmBuff
        lda     #36
_aseekop:
        inx
        cmp     AsmBuff+2,x             ; then AsmBuff+3,x to read operand
        bne     _aseekop

        cpy     #$09                    ; rel. addressing mode for branches
        beq     _asmrelative

        lda     Aux1                    ; instruction size
        cmp     #$02
        beq     _aoplow

        jsr     _asmgethex              ; get high byte
        bcs     _asmgeterr
        ldy     #$02
        sta     (CurrL),y               ; write high byte of 16-bit op.

_aoplow:
        jsr     _asmgethex
        bcs     _asmgeterr
_awritelo:
        ldy     #$01
        sta     (CurrL),y               ; write 8-bit or low byte of 16-bit

        lda     Aux1                    ; restore instruction size


;*      Successfully assembled one instruction

_adone:
        jsr     CurrAdv                 ; advance 'current address'
        jmp     _assemlp                ; continue assembling


;*      Calculate 8-bit signed offset for relative addressing mode

_asmrelative:
        jsr     _asmgethex              ; get high byte of relative op
        bcs     _asmgeterr
        sta     Aux4
        jsr     _asmgethex              ; get low byte of relative op
        bcs     _asmgeterr
        sec
        sbc     CurrL
        sta     Aux3
        lda     Aux4
        sbc     CurrH
        sta     Aux4

        bpl     _arelposchk

        lda     #$81
        cmp     Aux3
        bcs     _areltoolong
        lda     #$ff
        cmp     Aux4
        bne     _areltoolong

_arelok:
        ldx     Aux3
        dex
        dex
        txa

        jmp     _awritelo

_arelposchk:
        lda     #$81
        cmp     Aux3
        bcc     _areltoolong
        lda     #$00
        cmp     Aux4
        beq     _arelok
        ;
_areltoolong:
        lda     #33                     ; branch too long: "!?"
        jsr     CHROUT
        sec
        rts


;*      Get byte value from assemble buffer

_asmgethex:
        lda     AsmBuff+3,x             ; get high nybble
        inx
        jsr     FindNybble
        bcs     _asmgeterr

        rol
        rol
        rol
        rol
        and     #$f0
        sta     Aux5

        lda     AsmBuff+3,x             ; get low nybble
        inx
        jsr     FindNybble
        bcs     _asmgeterr

        ora     Aux5
        clc                             ; no error
_asmgeterr:
        rts


;****   Command D: Disassemble

Disassemble:
        jsr     GetAddr                 ; line: "D 1000"

        lda     #DisasmCount
        sta     Aux4

_disasmlp:
        jsr     CurrOut                 ; output: "1000 "
        jsr     SpaceOut

        ldy     #$02                    ; get opcode & possible operand
        lda     (CurrL),y
        sta     Aux2
        dey
        lda     (CurrL),y
        sta     Aux1
        dey
        lda     (CurrL),y               ; opcode
_seekopcode:
        cmp     Opcodes,y
        beq     _opcodefound

        iny
        cpy     #MnemPtr-Opcodes
        bne     _seekopcode

        lda     #$00                    ; unknown opcode
        tax
        beq     _disout                 ; always branch

_opcodefound:
        ldx.w   MnemPtr,y               ; get assembler symbol
        lda     AddrModes,y             ; get addressing mode
        ;
_disout:
        sta     Aux3                    ; store addressing mode

        ldy     #$03                    ; print assembler symbol
_opoutlp:
        lda.w   Assembly,x
        jsr     CHROUT
        inx
        dey
        bne     _opoutlp

        ldy     Aux3                    ; check addressing mode
        beq     _opdone                 ; branch if no operand

        jsr     SpaceOut

        lda     Prefixes,y              ; print prefix (always at least $)
        tay
        jsr     PrintMsg

        ldy     Aux3
        lda     OperTypes,y
        beq     _oprelative
        bmi     _opnot16

_relentry:
        lda     Aux2
        jsr     HexOut

_opnot16:
        lda     Aux1
        jsr     HexOut

        ldy     Aux3
        lda     Postfixes,y             ; print postfix if any
        beq     _opdone
        tay
        jsr     PrintMsg

        ldy     Aux3                    ; addressing mode
_opdone:
        lda     AddrSizes,y             ; get instruction size in bytes
        jsr     CurrAdv                 ; advance 'current address'

        jsr     STOP
        beq     _dasbreak               ; check stop key

        dec     Aux4
        bne     _disasmlp

_dasbreak:
        clc
        rts

_oprelative:
        sta     Aux2                    ; it's zero

        lda     Aux1
        bpl     _forw

        dec     Aux2                    ; $ff if backwards
_forw:
        clc
        adc     #$02
        bcc     _ornoc
        inc     Aux2
_ornoc:
        clc
        adc     CurrL
        sta     Aux1
        lda     CurrH
        adc     Aux2
        sta     Aux2
        jmp     _relentry


;****   Command T: Text view

TextView:
        jsr     GetAddr                 ; line: "T 1000"

        lda     #TextViewLines
        sta     Aux4

_textline:
        jsr     CurrOut                 ; output: "1000 ................"

        ldx     #TextLineWidth
        jsr     AscOut

        dec     Aux4
        bne     _textline
Success:
        clc                             ; also used for command <none>
        rts


;****   Command M: Hex dump

HexDump:
        jsr     GetAddr                 ; line: "M 1000"

        lda     #HexDumpLines
        sta     Aux4

_dumpline:
        jsr     CurrOut                 ; output: "1000 00 00 00 00 ...."

        ldy     #$00
_dump4:
        jsr     SpaceOut                ; " 00 00 00 00"

        tya
        pha
        lda     (CurrL),y
        jsr     HexOut
        pla
        tay

        iny
        cpy     #HexPerLine
        bne     _dump4

        ldx     #HexPerLine             ; " ...."
        jsr     AscOut

        dec     Aux4
        bne     _dumpline

        clc                             ; no error
        rts


;****   Command G: Go

JumpnGo:
        jsr     GetAddr                 ; line: "G 1000"
        bcs     _novalidaddr

        jsr     ReturnOut

        jsr     _gothere
        ;
Capture:
        php                             ; capture registers as they are
        sta.w   Regs+4
        stx.w   Regs+3
        sty.w   Regs+2
        tsx
        stx.w   Regs+1
        pla
        sta.w   Regs
        
        clc
        rts

_gothere:
        jmp     (CurrL)


;****   Command R: Registers

ShowRegisters:
        ldy     #$0c
        jsr     PrintMsg

        lda     #$04
        sta     Aux1

_nextreg:
        ldy     Aux1                    ; print captured ac, xr, yr, sp
        lda     Regs,y
        jsr     HexOut

        jsr     SpaceOut

        dec     Aux1
        bne     _nextreg

        lda.w   Regs                    ; get captured status register value
        sta     Aux2
        lda     #$08
        sta     Aux1

_nextbit:
        lda     #$00                    ; print status register bits
        rol     Aux2
        adc     #$30
        jsr     CHROUT

        dec     Aux1
        bne     _nextbit

        clc
        rts


;****   Command W: Write

WriteMem:
        jsr     GetAddr                 ; line: "W 1000 00 00 00 00 ..."
        bcs     _novalidaddr

        lda     #$00
        sta     Aux6
_writemem:
        jsr     CheckSpace
        bcs     _waslastbyte

        jsr     GetHex
        bcs     _novalidaddr            ; carry set if no valid hex

        ldy     Aux6
        sta     (CurrL),y
        inc     Aux6
        bne     _writemem

_waslastbyte:
        clc                             ; no error
_novalidaddr:
        rts


;****   Command S: Save memory

SaveMem:
        jsr     GetFilename             ; line: "S "name" 1000 1200"
        bcs     _smout

        jsr     GetTwoAddr
        bcs     _smout

        ldy     #$00
        jsr     SetupFile

        lda     #Aux1
        ldx     CurrL
        ldy     CurrH
        jsr     SAVE

        lda     Aux1
        sta     CurrL
        lda     Aux2
        sta     CurrH

        clc                             ; no error
_smout:
        rts


;****   Command L: Load memory

LoadMem:
        jsr     GetFilename             ; line: "L "name" 1000"
        bcs     _lmout

        jsr     GetAddr                 ; get load address
        ldy     #$00
        bcc     _noreloc

        iny                             ; secondary address for SETLFS
_noreloc:
        jsr     SetupFile

        lda     #$00                    ; load flag
        ldx     CurrL
        ldy     CurrH
        jsr     LOAD

        clc                             ; no error
_lmout:
        rts


;****   Get filename from command line

GetFilename:
        jsr     CheckSpace
        bcs     _noname

        jsr     CHRIN
        cmp     #34
        bne     _badname

        ldy     #$00
_namelp:
        jsr     CHRIN
        cmp     #13
        beq     _badname
        cmp     #34
        beq     _namend
        sta     Filename,y
        iny
        cpy     #$10
        bne     _namelp

_namend:
        sty     Aux3                    ; store filename length

        clc                             ; no error
_noname:
        rts

_badname:
        sec
        rts


;****   Set up file, y = secondary address

SetupFile:
        lda     #$01                    ; file number
        ldx     $ba                     ; current device
        jsr     SETLFS

        lda     Aux3                    ; filename length
        ldx     #Filename&255
        ldy     #Filename/256
        jsr     SETNAM

        rts


.IF COMPILE_LITE == 0

;****   Command F: Fill memory (not included in LITE version)

FillMem:
        jsr     GetTwoAddr              ; line: "F 1000 1100 10"
        bcs     _fnovalid

        jsr     CheckSpace
        bcs     _fnovalid

        jsr     GetHex                  ; get '10'
        bcs     _fnovalid

        ldy     #$00
_filllp:
        sta     (Aux1),y
        inc     Aux1
        bne     _fnoc1
        inc     Aux2
_fnoc1:
        ldx     Aux2                    ; check for last page
        cpx     CurrH
        bne     _filllp

        ldx     Aux1                    ; check for last byte
        cpx     CurrL
        bne     _filllp

        clc                             ; no error
_fnovalid:
        rts


;****   Command C: Copy memory (not included in LITE version)

CopyMem:
        jsr     GetTwoAddr              ; line: "C 1000 1100 1200"
        bcs     _cmnovalid

        sta     Aux3                    ; low byte still in a
        lda     CurrH
        sta     Aux4                    ; '1100' now in (Aux3)

        jsr     GetAddr                 ; get '1200'
        bcs     _cmnovalid

        ldy     #$00

        lda     Aux2
        cmp     CurrH                   ; Aux2 - CurrH
        bcc     _backwardcopy           ; backw if currh bigger
        bne     _forwardcopy            ; forw if currh smaller

        lda     Aux1
        cmp     CurrL
        bcc     _backwardcopy           ; branch is currl bigger

_forwardcopy:
        lda     (Aux1),y                ; copy forward
        sta     (CurrL),y

        inc     Aux1
        bne     _cnoc1
        inc     Aux2
_cnoc1:
        inc     CurrL
        bne     _cnoc2
        inc     CurrH
_cnoc2:
        lda     Aux2                    ; check for last page
        cmp     Aux4 
        bne     _forwardcopy

        lda     Aux1                    ; check for last byte
        cmp     Aux3
        bne     _forwardcopy

        clc                             ; no error
_cmnovalid:
        rts

_backwardcopy:
        lda     Aux3                    ; calculate size of block
        sec
        sbc     Aux1
        sta     Aux5
        lda     Aux4
        sbc     Aux2
        sta     Aux6

        lda     CurrL                   ; calculate end of destination area
        clc
        adc     Aux5
        sta     Aux5
        lda     CurrH
        adc     Aux6
        sta     Aux6

_backwlp:
        dec     Aux3                    ; copy backwards
        lda     Aux3
        cmp     #$ff
        bne     _cnoc3
        dec     Aux4
_cnoc3:
        dec     Aux5
        lda     Aux5
        cmp     #$ff
        bne     _cnoc4
        dec     Aux6
_cnoc4:
        lda     (Aux3),y
        sta     (Aux5),y

        lda     Aux4
        cmp     Aux2
        bne     _backwlp

        lda     Aux3
        cmp     Aux1
        bne     _backwlp

        clc
        rts


;****   Command @: Disk (Not included in LITE version)

DiskCmd:
        jsr     CHRIN                   ; line "@command" or "@" (status)
        cmp     #13
        bne     _dcommand

        jsr     ReturnOut

        ldy     #$0f                    ; command channel
        lda     #$00                    ; no filename
        sta     Aux3
        jsr     SetupFile

        jsr     OPEN
        bcs     _derror

        ldx     #$01                    ; input from channel 1
        jsr     CHKIN
        bcs     _derror

_derrout:
        jsr     CHRIN                   ; get and print disk status message
        jsr     CHROUT
        cmp     #13
        bne     _derrout

        jmp     _closeentry


_dcommand:
        ldy     #$00
_dcmdbuff:
        sta     Filename,y
        iny
        jsr     CHRIN
        cmp     #13
        bne     _dcmdbuff

        sty     Aux3
        ldy     #$0f                    ; command channel
        jsr     SetupFile

        jsr     OPEN                    ; send disk command
        bcs     _derror

        jmp     _closeentry


;****   Command $: Dir (not included in LITE version)

DiskDir:
        ldy     #$00                    ; secondary address

        lda     #$01                    ; filename
        sta     Aux3
        lda     #36
        sta.w   Filename

        jsr     SetupFile               ; prepare file
        jsr     OPEN
        bcs     _derror

        ldx     #$01                    ; input from channel 1
        jsr     CHKIN
        bcs     _derror

        jsr     CHRIN                   ; skip header
        jsr     CHRIN

_dirlp:
        jsr     _dirline
        bcc     _dirlp                  ; carry clear if not eof

_closeentry:
        jsr     _dclose
        clc                             ; no error
        rts

_derror:
        jsr     _dclose                 ; disk error
        sec
        rts

_dclose:
        lda     #$01                    ; close file
        jsr     CLOSE
        jsr     CLALL                   ; input back to keyboard
        rts

_dirline:
        jsr     CHRIN                   ; skip obsolete bytes
        jsr     CHRIN

        jsr     READST                  ; check end of file
        bne     _dirend

        jsr     ReturnOut

        jsr     STOP                    ; check stop key
        beq     _dirend

        jsr     CHRIN                   ; get and print number from file
        sta     $63
        jsr     CHRIN
        sta     $62
        jsr     DECOUT

        jsr     SpaceOut

_dirnamelp:
        jsr     CHRIN
        jsr     CHROUT
        bne     _dirnamelp

_direndline:
        clc
        rts

_dirend:
        sec
        rts


;****   Command N: Convert hex to decimal (not included in LITE version)

Convert:
        jsr     CheckSpace
        bcs     _baddrive

        jsr     GetHex
        bcs     _badhex
        sta     $62
        jsr     GetHex
        bcs     _badhex
        sta     $63

        ldy     #$0d
        jsr     PrintMsg

        jsr     DECOUT

        clc
_badhex:
        rts


;****   Command Z: Set drive # (not included in LITE version)

SetDrive:
        jsr     CheckSpace
        bcs     _baddrive

        jsr     GetHex
        bcs     _baddrive

        sta     $ba                     ; set 'last accessed device'
        clc
_baddrive:
        rts

.ENDIF


;****   Get two addresses from command line

GetTwoAddr:
        jsr     GetAddr
        bcs     _gtnovalid

        sta     Aux1                    ; low byte still in a
        lda     CurrH
        sta     Aux2                    ; '1000' now in (Aux1)

        jsr     GetAddr                 ; get '1100'
        bcs     _gtnovalid

        clc
_gtnovalid:
        rts


;****   Check space in command line

CheckSpace:
        jsr     CHRIN
        and     #$7f                    ; recognize shift+space too
        cmp     #32
        bne     _notspace
        clc
        rts
_notspace:
        sec
        rts


;****   Print current address on a new line

CurrOut:
        jsr     ReturnOut

        lda     CurrH
        jsr     HexOut
        lda     CurrL
        jsr     HexOut

        rts


;****   Advance current address by a

CurrAdv:
        clc
        adc     CurrL
        sta     CurrL
        bcc     _noc
        inc     CurrH
_noc:
        rts


;****   Dump x bytes of text starting from current address

AscOut:
        jsr     SpaceOut

        ldy     #$00

_nextasc:
        lda     (CurrL),y
        cmp     #$a0
        bcs     _ascit
        cmp     #$80
        bcs     _badasc
        cmp     #$20
        bcc     _badasc

_ascit:
        jsr     CHROUT
        iny
        dex
        bne     _nextasc

        tya
        jsr     CurrAdv

        rts

_badasc:
        lda     #46             ; dot for unprintable characters
        bne     _ascit          ; always branch


;****   Output return

ReturnOut:
        lda     #13
        bne     _commonout      ; always branch

;****   Output space

SpaceOut:
        lda     #32
_commonout:
        jsr     CHROUT
        rts


;****   Print a in hex

HexOut:
        tax
        ror                     ; write high nybble
        ror
        ror
        ror
        and     #$0f
        tay
        lda     Hex,y
        jsr     CHROUT

        txa
        and     #$0f            ; write low nybble
        tay
        lda     Hex,y
        jsr     CHROUT

        rts


;****   Get new 'current address' from command line

GetAddr:
        jsr     CheckSpace
        bcs     _noaddr

        jsr     GetHex                  ; read new address from cmd line
        bcs     _noaddr                 ; carry set if no valid hex
        sta     CurrH

        jsr     GetHex
        bcs     _noaddr                 ; carry set if no valid hex
        sta     CurrL

        clc                             ; carry clear if valid address
_noaddr:
        rts


;****   Get byte value from command line (in hex)

GetHex:
        jsr     CHRIN                   ; get high nybble
        jsr     FindNybble
        bcs     _nothex

        rol
        rol
        rol
        rol
        and     #$f0
        sta     Aux5

        jsr     CHRIN                   ; get low nybble
        jsr     FindNybble
        bcs     _nothex

        ora     Aux5
        clc                             ; no error
_nothex:
        rts


;****   Convert ascii nybble to binary

FindNybble:
        ldy     #$0f
_seek:
        cmp     Hex,y
        beq     _found
        dey
        bpl     _seek

        sec                             ; not hex
        rts
_found:
        tya
        clc                             ; no error
        rts

.IF COMPILE_LITE == 0

;****   Command H or ?: Show help

ShowHelp:
        ldy     #$03
        ;
.ENDIF
        ;
;****   Print message number y
PrintMsg:
        lda     MsgL,y
        sta     Aux5
        lda     MsgH,y
        sta     Aux6

        ldy     #$00
_prnc:
        lda     (Aux5),y
        beq     _prnted
        jsr     CHROUT
        iny
        bne     _prnc
_prnted:
        clc
        rts


;****   Command list and jump table

CmdList:
        .DB     13
        .DB     "A"
        .DB     "M"
        .DB     "I"
        .DB     "W"
        .DB     "G"
        .DB     "D"
        .DB     "R"
        .DB     "S"
        .DB     "L"

.IF COMPILE_LITE == 0
        .DB     "Z"                     ; not included in LITE version
        .DB     "C"
        .DB     "N"
        .DB     "$"
        .DB     "@"
        .DB     "?"
        .DB     "H"
        .DB     "F"
.ENDIF

CmdJumpL:
        .DB     Success&255
        .DB     Assemble&255
        .DB     HexDump&255
        .DB     TextView&255
        .DB     WriteMem&255
        .DB     JumpnGo&255
        .DB     Disassemble&255
        .DB     ShowRegisters&255
        .DB     SaveMem&255
        .DB     LoadMem&255

.IF COMPILE_LITE == 0
        .DB     SetDrive&255            ; not included in LITE version
        .DB     CopyMem&255
        .DB     Convert&255
        .DB     DiskDir&255
        .DB     DiskCmd&255
        .DB     ShowHelp&255
        .DB     ShowHelp&255
        .DB     FillMem&255
.ENDIF

CmdJumpH:
        .DB     Success/256
        .DB     Assemble/256
        .DB     HexDump/256
        .DB     TextView/256
        .DB     WriteMem/256
        .DB     JumpnGo/256
        .DB     Disassemble/256
        .DB     ShowRegisters/256
        .DB     SaveMem/256
        .DB     LoadMem/256
.IF COMPILE_LITE == 0
        .DB     SetDrive/256            ; not included in LITE version
        .DB     CopyMem/256
        .DB     Convert/256
        .DB     DiskDir/256
        .DB     DiskCmd/256
        .DB     ShowHelp/256
        .DB     ShowHelp/256
        .DB     FillMem/256
.ENDIF


;****   Assembler tables

.IF UNDOCUMENTED == 1
Opcodes:
        .INCBIN "x_opcode.bin"          ; 6502/6510 instruction set
MnemPtr:                                ; including undocumented commands
        .INCBIN "x_mneptr.bin"
AddrModes:
        .INCBIN "x_adrmod.bin"
Assembly:
        .INCBIN "x_assemb.bin"
.ELSE
Opcodes:
        .INCBIN "n_opcode.bin"          ; 6502/6510 instruction set
MnemPtr:                                ; without undocumented commands
        .INCBIN "n_mneptr.bin"
AddrModes:
        .INCBIN "n_adrmod.bin"
Assembly:
        .INCBIN "n_assemb.bin"
.ENDIF


AddrSizes:
        .DB     $01,$02,$02,$02,$03,$03,$03,$02,$02,$02,$03,$02

OperTypes:
        .DB     $80,$80,$80,$80,$01,$01,$01,$80,$80,$00,$01,$80

Prefixes:
        .DB     $00,$04,$05,$05,$05,$05,$05,$06,$06,$05,$06,$05

Postfixes:
        .DB     $00,$00,$00,$07,$00,$07,$08,$09,$0a,$00,$0b,$08

AsmMaskPtr:
        .DB     0
        .DB     am1-AsmMasks
        .DB     am2-AsmMasks
        .DB     am3-AsmMasks
        .DB     am4-AsmMasks
        .DB     am5-AsmMasks
        .DB     am6-AsmMasks
        .DB     am7-AsmMasks
        .DB     am8-AsmMasks
        .DB     am4-AsmMasks    ; 9 looks same as 4
        .DB     ama-AsmMasks
        .DB     amb-AsmMasks


;****   Messages

MsgL:   .DB     MsgWelcome&255          ; $00 (welcome)
        .DB     (MsgError+1)&255        ; $01 (ready)
        .DB     MsgError&255            ; $02 (error)
        .DB     MsgHelp&255             ; $03 (help)

        .DB     PreA&255                ; $04 "#$"
        .DB     (PreA+1)&255            ; $05 "$"
        .DB     PreB&255                ; $06 "($"

        .DB     PostA&255               ; $07 ",X"
        .DB     (PostC+1)&255           ; $08 ",Y"
        .DB     PostB&255               ; $09 ",X)"
        .DB     PostC&255               ; $0a "),Y"
        .DB     (PostB+2)&255           ; $0b ")"

        .DB     MsgRegister&255         ; $0c (registers)
        .DB     MsgConv&255             ; $0d (hex conversion)

MsgH:   .DB     MsgWelcome/256
        .DB     (MsgError+1)/256
        .DB     MsgError/256
        .DB     MsgHelp/256

        .DB     PreA/256                ; $04 "#$"
        .DB     (PreA+1)/256            ; $05 "$"
        .DB     PreB/256                ; $06 "($"

        .DB     PostA/256               ; $07 ",X"
        .DB     (PostC+1)/256           ; $08 ",Y"
        .DB     PostB/256               ; $09 ",X)"
        .DB     PostC/256               ; $0a "),Y"
        .DB     (PostB+2)/256           ; $0b ")"

        .DB     MsgRegister/256
        .DB     MsgConv/256


AsmMasks:
        .DB     13              ; $00 Implied/Accu
am1:
        .DB     " #$00",13      ; $01 Immediate
am2:
        .DB     " $00",13       ; $02 Zero page
am3:
        .DB     " $00,X",13     ; $03 Zero page,x
am4:
        .DB     " $0000",13     ; $04 Absolute
am5:
        .DB     " $0000,X",13   ; $05 Absolute,x
am6:
        .DB     " $0000,Y",13   ; $06 Absolute,y
am7:
        .DB     " ($00,X)",13   ; $07 (Indirect,x)
am8:
        .DB     " ($00),Y",13   ; $08 (Indirect),y
;am9:
;        .DB     " $0000",13     ; $09 Relative
ama:
        .DB     " ($0000)",13   ; $0a Indirect
amb:
        .DB     " $00,Y",13     ; $0b Zero page,y


Hex:
        .DB     "0123456789ABCDEF"

MsgError:
        .DB     "?",13,".",0

PreA:   .DB     "#$",0
PreB:   .DB     "($",0

PostA:  .DB     ",X",0
PostB:  .DB     ",X)",0
PostC:  .DB     "),Y",0

MsgRegister:
        .DB     13,"AC XR YR SP NV-BDIZC",13,0


.IF COMPILE_LITE == 0
MsgConv:
        .DB     " = DEC ",0

MsgHelp:
        .DB     13,"ASM  A 1000"
        .DB     13,"COPY C 1000 1200 1E00"
        .DB     13,"FILL F 1000 1200 10"
        .DB     13,"GO   G 1000"
        .DB     13,"WRIT W 1000 10 20 30",13

        .DB     13,"DASM D 1000"
        .DB     13,"TEXT I 1000"
        .DB     13,"DUMP M 1000"
        .DB     13,"CONV N 1000"
        .DB     13,"REGS R",13

        .DB     13,"DIR  $"
        .DB     13,"CMD  @S:FILE"
        .DB     13,"LOAD L ",34,"FI",34," 1000"
        .DB     13,"SAVE S ",34,"FI",34," 1000 1200"
        .DB     13,"DRV  Z 08",0

MsgWelcome:
        .IF COMPILE_C64 == 0
        .DB     13,"C'MON V1.0 BY A.EEBEN",13
        .DB     "(SYS1024) H FOR HELP",13,".",0
        .ELSE
        .DB     13,"C'MON V1.0/C64 BY A.EEBEN (SYS50176)",13
        .DB     "TYPE H FOR COMMAND HELP",13,".",0
        .ENDIF
.ELSE
MsgConv:
MsgHelp:
MsgWelcome:
        .IF COMPILE_C64 == 0
        .DB     13,"C'MON LITE BY A.EEBEN",13
        .DB     "(SYS5632) C:ADGILMRSW",13,".",0
        .ELSE
        .DB     13,"C'MON LITE/C64 BY A.EEBEN (SYS50944)",13
        .DB     "LITE VERSION COMMANDS: ADGILMRSW",13,".",0
        .ENDIF
.ENDIF

.ENDS
